library ieee;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity handy is
    port (
        -- eingänge
        i_ruf: in std_logic;
        i_ziffer_wahl: in std_logic;
        i_nummer_unvollstaendig: in std_logic;
        i_verbunden: in std_logic;
        i_verbinden: in std_logic;
        i_besetzt: in std_logic;
        i_abbruch: in std_logic;
        i_auflegen: in std_logic;
        i_antwort: in std_logic;
        -- ausgänge
        o_besetzt_ton: out std_logic;
        o_klingel_ton: out std_logic;
        -- synchrones Schaltwerk
        i_reset: in std_logic;
        i_clk: in std_logic
        );
end handy;
architecture arch_HANDY of handy is
    -- interne signale
    type state_type is (IDLE,WAHL,WAEHLEN,SPRECHEN,KLINGELN,ABBRUCH,VERBINDEN,BESETZT);
    signal cur_state:state_type;
    signal next_state:state_type;
begin
    Updater:process(i_clk,i_reset)
    begin
        if (i_reset = '1') then
            cur_state <= IDLE;
        elsif (rising_edge(i_clk)) then
            cur_state <= next_state;
        end if;
    end process Updater;

    FSM:process(i_ruf, i_ziffer_wahl, i_nummer_unvollstaendig, i_verbunden,
        i_besetzt, i_abbruch, i_auflegen, i_antwort, i_verbinden)
        variable tmp_o_besetzt_ton:std_logic := '0';
        variable tmp_o_klingel_ton:std_logic := '0';
    begin
        tmp_o_besetzt_ton := '0';
        tmp_o_klingel_ton := '0';
        case (cur_state) is
            when IDLE =>
                if (i_ruf = '1') then
                    next_state <= WAHL;
                elsif (i_abbruch = '1') then
                    next_state <= ABBRUCH;
                else
                    next_state <= IDLE;
                end if;
            when WAHL =>
                if (i_ziffer_wahl = '1') then
                    next_state <= WAEHLEN;
                elsif (i_abbruch = '1') then
                    next_state <= ABBRUCH;
                else
                    next_state <= WAHL;
                end if;
            when WAEHLEN =>
                if (i_nummer_unvollstaendig = '1') then
                    next_state <= WAEHLEN;
                elsif (i_verbinden = '1') then
                    next_state <= VERBINDEN;
                elsif (i_abbruch = '1') then
                    next_state <= ABBRUCH;
                else
                    next_state <= WAEHLEN;
                end if;
            when ABBRUCH =>
                    next_state <= IDLE;
            when SPRECHEN =>
                if (i_auflegen = '1') then
                    next_state <= IDLE;
                elsif (i_abbruch = '1') then
                    next_state <= ABBRUCH;
                else
                    next_state <= SPRECHEN;
                end if;
            when KLINGELN =>
                if (i_antwort = '1') then
                    next_state <= SPRECHEN;
                elsif (i_abbruch = '1') then
                    next_state <= ABBRUCH;
                else
                    next_state <= KLINGELN;
                end if;
            when VERBINDEN =>
                if (i_besetzt = '1') then
                    next_state <= BESETZT;
                    tmp_o_besetzt_ton := '1';
                elsif (i_verbunden = '1') then
                    next_state <= KLINGELN;
                    tmp_o_klingel_ton := '1';
                elsif (i_abbruch = '1') then
                    next_state <= ABBRUCH;
                else
                    next_state <= VERBINDEN;
                end if;
            when BESETZT =>
                if (i_ruf = '1') then
                    next_state <= WAHL;
                elsif (i_abbruch = '1') then
                    next_state <= ABBRUCH;
                else
                    next_state <= BESETZT;
                end if;
        end case;
        o_besetzt_ton <= tmp_o_besetzt_ton;
        o_klingel_ton <= tmp_o_klingel_ton;
    end process FSM;
end architecture;

